/* 1. setup database */
var mongoose = require('mongoose');
mongoose.connect('mongodb://localhost/products');

var Schema = mongoose.Schema;
var ProductSchema = new Schema({
	name: {type: String, required: true, unique: true},
	unitprice: {type: Number, required: true},
	description: {type: String, trim: true},
	discontinued: {type: Boolean}
});
	
var Product = mongoose.model('Product', ProductSchema);
/* 2. setup the express framework, you can install by: npm install express */
var express = require('express');
var app = express();
/* from the docs:
 * Request body parsing middleware supporting JSON, urlencoded, and multipart requests. 
 * This middleware is simply a wrapper the json(), urlencoded(), and multipart() middleware.
 */
app.use(express.bodyParser());
/* simulate DELETE and PUT methods */
app.use(express.methodOverride());

/* REST CRUD API */
/* Retrieve all products */
app.get('/', function (req, res) {
	Product.find({}, function(error, data) {
		res.json(data);
	});
});
/* Get Product by ID */
app.get('/:id', function (req, res) {
	Product.find({_id: req.params.id}, function (error, data) {
		res.json(data);
	});
});
/* Create new Products */
app.post('/new', function (req, res) {
	var product_data = {
		name: req.body.name,
		unitprice: req.body.unitprice,
		description: req.body.description,
		discontinued: req.body.discontinued
	};
	var product = new Product(product_data);
	product.save(function (error, data) {
		if (error) {
			res.json(error);
		} else {
			console.log("Added New Product");
			res.statusCode = 201;
			res.send();
		}
	});
});
/* Update an existing Product by ID */
app.put('/update/:id', function (req, res) {
	Product.update(
		{_id:req.params.id},
		{
			$set: {
				name: req.body.name,
				description: req.body.description,
				unitprice: req.body.unitprice,
				description: req.body.description,
				discontinued: req.body.discontinued
			}
		},
		false,
		true
	);
	console.log("Updated Existing Product with ID: " + req.params.id);
	res.send();        
});
/* Delete Product by ID */
app.delete('/delete/:id', function (req, res) {
	return Product.findById(req.params.id, function (error, product) {
		return product.remove(function (error) {
			if (error) {
				console.log(error);
			} else {
				console.log("deleted product: " + req.params.id);
				return res.send();
			}
		});
	});
});
app.listen(8082);
console.log("Server running at localhost:8082");